@@ -1521,23 +1521,12 @@ Will use HTML5 for this SASsession.""")
       if encode_errors is None:
          encode_errors = 'fail'
 
-      bpc = self._sb.pyenc[0]
-      if char_lengths and str(char_lengths).strip() in ['1','2','3','4']:
-         bpc = int(char_lengths)
-      if char_lengths and str(char_lengths) == 'exact':
-         CnotB = False
-      else:
-         CnotB = bpc == 1
+      CnotB = kwargs.pop('CnotB', None)
 
-      if type(char_lengths) is not dict or len(char_lengths) < ncols:
-         charlens = self._sb.df_char_lengths(df, encode_errors, char_lengths)
-      else:
-         charlens = char_lengths
-
-      if charlens is None:
+      if char_lengths is None:
          return -1
 
-      chr_upper = {k.upper():v for k,v in charlens.items()}
+      chr_upper = {k.upper():v for k,v in char_lengths.items()}
 
       if type(df.index) != pd.RangeIndex:
          warnings.warn("Note that Indexes are not transferred over as columns. Only actual columns are transferred")
@@ -1545,7 +1534,7 @@ Will use HTML5 for this SASsession.""")
       for name in df.columns:
          colname = str(name).replace("'", "''")
          col_up  = str(name).upper()
-         input  += "'"+colname+"'n "
+         input  += "input '"+colname+"'n "
          if col_up in lab_keys:
             label += "label '"+colname+"'n ="+lab_upper[col_up]+";\n"
          if col_up in fmt_keys:
@@ -1592,6 +1581,7 @@ Will use HTML5 for this SASsession.""")
                   dts.append('B')
                else:
                   dts.append('N')
+         input += ';\n'
 
       code = "data "
       if len(libref):
@@ -1611,8 +1601,11 @@ Will use HTML5 for this SASsession.""")
       if len(format):
          code += "format "+format+";\n"
       code += label
-      code += "infile datalines delimiter="+delim+" STOPOVER;\ninput @;\nif _infile_ = '' then delete;\n"
-      code += "else do;\n input "+input+";\n"+xlate+";\nend;\ndatalines4;"
+      code += "infile datalines delimiter="+delim+" STOPOVER;\n"
+      code += "input @;\nif _infile_ = '' then delete;\nelse do;\n"
+      code +=  input+xlate+";\n"
+      code += "end;\n"
+      code += "datalines4;"
       self._asubmit(code, "text")
 
       blksz = int(kwargs.get('blocksize', 32767))
@@ -1629,7 +1622,7 @@ Will use HTML5 for this SASsession.""")
                var = '.'
             elif dts[col] == 'C':
                if var == 'nan' or len(var) == 0:
-                  var = ' '
+                  var = ' '+colsep
                else:
                   var = var.replace(colsep, colrep)
             elif dts[col] == 'B':
@@ -1640,15 +1633,13 @@ Will use HTML5 for this SASsession.""")
                else:
                   var = str(row[col].to_datetime64())[:26]
 
-            card += var
-            if col < (ncols-1):
-               card += colsep
+            if embedded_newlines:
+               var = var.replace(LF, colrep).replace(CR, colrep)
+               var = var.replace('\n', LF).replace('\r', CR)
 
-         if embedded_newlines:
-            card = card.replace(LF, colrep).replace(CR, colrep)
-            card = card.replace('\n', LF).replace('\r', CR)
+            card += var+"\n"
 
-         code += card+"\n"
+         code += card
 
          if len(code) > blksz:
             if not noencode:
@@ -1682,8 +1673,12 @@ Will use HTML5 for this SASsession.""")
          else:
             code = code.encode(self.sascfg.encoding, errors='replace').decode(self.sascfg.encoding)
 
-      self._asubmit(code+";;;;\n;;;;", "text")
+      self._asubmit(code, "text")
+      self._asubmit(";;;;\n;;;;", "text")
       ll = self.submit("quit;", 'text')
+      if ("We failed in Submit" in ll['LOG']):
+         logger.error("Failure in the IOM client code, likely a transcoding error encountered. Data transfer stopped on or before row "+str(row_num))
+         logger.error("Rendering the error from the Java layer:\n\n"+ll['LOG'].partition("END We failed in Submit\n")[0])
       return None
 
    def sasdata2dataframe(self, table: str, libref: str ='', dsopts: dict = None,
@@ -2082,8 +2077,10 @@ Will use HTML5 for this SASsession.""")
                if i % 10 == 9:
                   code +='\n'
 
+      lreclx = max(self.sascfg.lrecl, (lrecl + nvars + 1))
+
       miss  = {}
-      code += "\nfile "+self._tomods1.decode()+" lrecl="+str(self.sascfg.lrecl)+" dlm="+cdelim+" recfm=v termstr=NL encoding='utf-8';\n"
+      code += "\nfile "+self._tomods1.decode()+" lrecl="+str(lreclx)+" dlm="+cdelim+" recfm=v termstr=NL encoding='utf-8';\n"
       for i in range(nvars):
          if vartype[i] != 'N':
             code += "'"+varlist[i]+"'n = translate('"
